function [phi,ivs,psi,pvs,phir,ivsr,phikk,phikkr]=PredCoef(r0,r,s0,s,m,h,K)
% 
% given the covariance function of an m-dim times series forms forward and backward  one
% step prediction coefficients and prediction error variances from its own past using the Whittle recursions
% also, if h>0, forms prediction coefficients and error variances for a further h-dim series from the
% past of the given series from its cross-covariances with that series
% set h=0 if no other series relevant: then s0 and s can be set to zero and psi and pvs are returned as zero
% r0, r are the mxm variance and auto-covariances from lag 1 to at least K of the given series (m-dim) x
% s0, s are the hxh variance and hxm cross-covariances from lag 1 to at least K of the further series (h-dim) y with x
% phi are -ve (m,m) lag K prediction coefficients of x on its lagged (1 to K) values, prefixed by I
% phikk are lag k prediction coefficients for order k = 1 to K predictors, prefixed by 0
% phir are -ve (m,m) lag K backward prediction coefficients of x on its lagged (1 to K) values, prefixed by I
% phikkr are lag k prediction coefficients for order k = 1 to K backward predictors prefixed by 0
% ivs are innovation variances of x for AR orders (1 to K), prefixed by r0. 
% ivsr are backward innovation variances of x for AR orders (1 to K), prefixed by r0. 
% psi are (hxm) prediction coefficients of variable y on lagged (1 to K) values of x prefixed by zero
% pvs are (hxh) prediction error variances of y on lagged values of x (1 to K) prefixed by s0
%
% disp('Release version of PredCoef')
%
zx=zeros(m,m,(K+1));
phi=zx; phir=zx; ivs=zx; ivsr=zx; % initialise predictor coefficients for x
phikk=zx;phikkr=zx;
phi(:,:,1)=eye(m); phir(:,:,1)=eye(m);
ivsud=r0; ivsrud=r0; % initialise updated variables for x prediction error variances
xcovud=zeros(m,m); % initialise updated covariance variable for x
xcoefud=zeros(m,m); xcoefrud=zeros(m,m); % initialise updated coefficient variables for x
ivs(:,:,1)=ivsud; ivsr(:,:,1)=ivsrud; % initialise innovation variances

if h>0
    zy=zeros(h,h,(K+1));zyx=zeros(h,m,(K+1));
    psi=zyx; pvs=zy; % initialise predictor coefficients for y
    ycovud=zeros(h,h); % initialise updated covariance variable for y    
    ycoefud=zeros(h,m); % initialise updated coefficient variable for y   
    pvsud=s0; % initialise updated variable for y prediction error variance
    pvs(:,:,1)=pvsud; % initialise innovation variances for y
else
    pvsud=0;
    psi=0;
    pvs=0;  
end

if K>0
    % recursive loop begins 
    for i=1:K
        xcovud=0; xcovrud=0; ycovud=0;
        for j=1:i
            xcovud=xcovud+r(:,:,i+1-j)*phir(:,:,j)'; % new covariance variable for x
            if h>0
                ycovud=ycovud+s(:,:,i+1-j)*phir(:,:,j)'; % new covariance variable for y
            end
        end
        xcoefud=xcovud/ivsrud; % new coefficient for predictor of x
        phikk(:,:,i+1)=xcoefud;
        xcoefrud=xcovud'/ivsud; % new coefficient for reversed predictor of x
        phikkr(:,:,i+1)=xcoefrud;
        if h>0
            ycoefud=ycovud/ivsrud; % new coefficient for predictor of y
        end
        ivsud=ivsud-xcovud*xcoefud';  % new prediction error variance for x
        ivs(:,:,i+1)=ivsud;
        ivsrud=ivsrud-xcovud'*xcoefrud';  % new reversed prediction error variance for x
        ivsr(:,:,i+1)=ivsrud;
        
%             i
% %           Internal check: 
% disp('the following should all be the same for each recursion step i')
%             ratf=det(ivs(:,:,i+1))/det(ivs(:,:,i))
%             ratr=det(ivsr(:,:,i+1))/det(ivsr(:,:,i))
%             fac=det(eye(m)-xcoefud*xcoefrud)
% disp('and the following should all be less than 1 for each recursion step i')        
%             aef=abs(eig(xcoefud))
%             aer=abs(eig(xcoefrud))
        
        if h>0
            pvsud=pvsud-ycovud*ycoefud';  % new prediction error variance for y
            pvs(:,:,i+1)=pvsud;
        end
        for j=1:i+1
            jc=i+2-j;
            hold=phi(:,:,j); holdr=phir(:,:,jc); 
            phi(:,:,j)=hold-xcoefud*holdr; % update coefficients of predictor of x
            phir(:,:,jc)=holdr-xcoefrud*hold; % update coefficients of reverse predictor of x
            if h>0
                psi(:,:,j)=psi(:,:,j)+ycoefud*holdr; % update coefficients of predictor of y
            end
        end
    end
end



