function [stestfil,stestsmo,stvarfil,stvarsmo] =SSfilsmoMV(x,H,T,G,P)
% State Space model filtering and smoothing of non-time-varying model
% using square root methods for stable forward and backward computations
% estimates filtered states and smoothed states with their variance matrices
% for m by n series x, which can incorporate missing values set as NaN, 
% represented by a state space model with observation and transition matrices H,T
% and disturbance variance matrix G'G supplied via G
% with no observation noise (which could be incorporated into states). 
% Optionally the stationary state variance matrix P'P can be supplied via P
% else it is approximated in the function
% stestfil is estimate given all observations up to and including that time
% stestsmo is estimate given all observations 
% stvarfil is filtered state estimate variance matrix
% stvarsmo is smoothed state estimate variance matrix
%
% disp('Release version of SSfilsmoMV')
%
[m,n]=size(x); % series dimension and length
[d,dd]=size(T); % state dimension
[mm,ddd]=size(H); % observation map
[b,dddd]=size(G);
if (m~=mm)|(d~=dd)|(d~=ddd)|(d~=dddd)
    disp('non-conformable parameters')
    return
end
stestfil=zeros(d,n);
stestsmo=zeros(d,n);
stvarfil=zeros(d,d,n);
stvarsmo=zeros(d,d,n);

% the following are saved on the filtering pass to be used on the smoothing pass
N=zeros(d,d,n); % transposed regression of estimated state t-1 upon estimated state t
D=zeros(d,d,n); % right factor of reduction in variance from filtered to smoothed state at time t
J=zeros(d,d,n); % collected rows of Kalman gains at each time
sub1=[1:d];
sub2=d+sub1;

% obtain initial state variance chol factor P
if nargin==4
    nrpts=10; % 2^nrpts is initialisation period
    Tp=T;
    P=G;
    for rpts=1:nrpts
        [Q,P]=qr([P*Tp';P],0);
        Tp=Tp*Tp;
    end
end
% P,G
% StatVar=P'*P
eps=1.e-7; % tolerance for exactly known state variable
lam=1.0; % stabilising factor for singular state variance
veceps=eps*abs(diag(P));
veclam=lam*diag(P);

% P % is right factor of stationary state variance matrix
% del=P'*P-T*P'*P*T'-G'*G % check on solution to Lyapunov equations
%
% initialise state vector 
Sest=zeros(d,1);
% Filtering loop
for t=1:n
    Pold=P;  
    Sest=T*Sest; % predicted state from previous estimated state
    A=[P*T';G]; % state variance right factor update augmentation
    [Q,P]=qr(A,0); % right factor P of predicted state at time t
    
    % evaluate regression predictor of state at time t-1 from state at time t
    % for use in backward smoother
    B=[A,[Pold;G*0]];
    [Q2,R2]=qr(B,0);
    M=R2(sub1,sub1);
    Mc=M;
    for i=1:d
        if abs(M(i,i))<veceps(i)
            Mc(i,i)=veclam(i); % remove singularity
        end
    end
    Mi=inv(Mc);
    for i=1:d
        if abs(M(i,i))<veceps(i)
            Mi(i,i)=0; % restore to ensure correct singular prediction
        end
    end
    R2t=R2(sub1,sub2);
    Nt=Mi*R2t;
    N(:,:,t)=Nt; % transposed regression coeff computed
    % 
    % now forward update state at time t
    for i=1:m
        if ~isnan(x(i,t))
            xobs=x(i,t); % current observation
            Hrow=H(i,:);
            Xpred=Hrow*Sest;
            f=P*Hrow';
            [L,C]=hhrv(f);
            R=P-2*L*(L'*P); % application of reflection to P
            K=R(1,:); % Kalman gain as applied to standardised errors
            J(i,:,t)=K; % collect Kalman gains for backward smoother
            R(1,:)=0;
            P=R; % right factor of updated state variance (first row zeros could be removed)
            Sest=Sest + (K')*(xobs-Xpred)/C;
%             if t==1
%                 K/C
%             end
        end
    end
    stestfil(:,t)=Sest; % updated state estimate at time t
    stvarfil(:,:,t)=P; % updated state variance factor at time t
end

% backward smoother
stestsmo(:,n)=stestfil(:,n);
D(:,:,n)=0;
for t=n-1:-1:1
    stestsmo(:,t)=stestfil(:,t)+N(:,:,t+1)'*(stestsmo(:,t+1)-T*stestfil(:,t));
    A=[D(:,:,t+1);J(:,:,t+1)];
    [Q3,R3]=qr(A,0);
    D(:,:,t)=R3*N(:,:,t+1);
end
for t=1:n
    P=stvarfil(:,:,t);
    stvarfil(:,:,t)=P'*P;
    stvarsmo(:,:,t)=stvarfil(:,:,t)-D(:,:,t)'*D(:,:,t);
end

return


